#######################################################################
##### Part of the VortexGE Configuration Script                   #####
##### Copyright (C) 2004-2006 Aloysius Indrayanto                 #####
##### GNU Lesser General Public License                           #####
##### ----------------------------------------------------------- #####
##### EDITING THIS FILE ERRONEOUSLY MAY CAUSE THIS SCRIPT TO FAIL #####
##### ----------------------------------------------------------- #####
##### This program is distributed WITHOUT ANY WARRANTY.           #####
##### See the GNU Lesser General Public License for more details. #####
#######################################################################

#######################################################################
##### Constants Used by Dialog TUI
#######################################################################
DlgCmd=''
DAnsFl="dialog.$$"
DTitle="VortexGE $LVER Configuration Script - Copyright (C) 2004-2006 Aloysius Indrayanto"
DWidth=60
DCntIM=10


#######################################################################
##### Search for Usable Dialog Program
#######################################################################
if [[ -x `which dialog` ]]; then
  DlgCmd=`which dialog`
elif [[ -x `which cdialog` ]]; then
  DlgCmd=`which cdialog`
elif [[ -x `which Xdialog` ]]; then
  DlgCmd=`which Xdialog`
fi


#######################################################################
##### Functions for User Interaction Using Dialog TUI
#######################################################################

##### Check if the user press cancel
CheckCancel()
{
  if (( $? != 0 )); then
   rm -f $DAnsFl
   echo
   echo $_COLOR_RED"Aborted!"$_COLOR_NORMAL
   echo
   ExitScript 1;
  fi
}

##### Ask for some text using input box
DialogInputBox()
{
  $DlgCmd --backtitle "$DTitle" --inputbox "$1" 9 $DWidth "$2" 2> $DAnsFl
  CheckCancel
}

##### Ask for a choice using menu box
DialogMenuBox()
{
  local BTitle=$1
  local ICount=$#
  local Height;

  let 'ICount=(ICount - 1)/2'
  if (( ICount > DCntIM )); then ICount=$DCntIM; fi

  let 'Height=ICount + 8'

  shift 1
  $DlgCmd --backtitle "$DTitle" --menu "$BTitle" $Height $DWidth $ICount "$@" 2> $DAnsFl
  CheckCancel
}

##### Ask for some options using check box
DialogCheckBox()
{
  local BTitle=$1
  local ICount=$#
  local Height;

  let 'ICount=(ICount - 1)/3'
  if (( ICount > DCntIM )); then ICount=$DCntIM; fi

  let 'Height=ICount + 8'

  shift 1
  $DlgCmd --backtitle "$DTitle" --separate-output --checklist "$BTitle" $Height $DWidth $ICount "$@" 2> $DAnsFl
  CheckCancel
}

##### Ask for an option using radio box
DialogRadioBox()
{
  local BTitle=$1
  local ICount=$#
  local Height;

  let 'ICount=(ICount - 1)/3'
  if (( ICount > DCntIM )); then ICount=$DCntIM; fi

  let 'Height=ICount + 8'

  shift 1
  $DlgCmd --backtitle "$DTitle" --radiolist "$BTitle" $Height $DWidth $ICount "$@" 2> $DAnsFl
  CheckCancel
}

##### Get the user answer
DialogGetAns()
{
  local __ans__=`cat $DAnsFl`
  export $1="$__ans__"
}

##### Interactively ask the user for some options using dialog TUI
DoDialogTUI()
{
  ##### Ask for the installation directory
  DialogInputBox "Installation directory" "$VGInsDDef"
  DialogGetAns VGInsDSel
  FixDir $VGInsDSel VGInsDSel

  ##### Ask for processor type to be used
  local __cpu__
  DialogMenuBox "Supported CPUs"                        \
    0 "Autodetect                                     " \
    1 "Any i586 Compatible                            " \
    2 "Any i686 Compatible                            " \
    3 "Intel's CPUs                                  >" \
    4 "AMD's CPUs                                    >" \
    5 "Other Vendors' CPUs                           >"
  DialogGetAns __cpu__
    if [[ $__cpu__ ==  '0' ]]; then Processor=$CPU_Autodetect
  elif [[ $__cpu__ ==  '1' ]]; then Processor=$CPU_Anyi586Compatible
  elif [[ $__cpu__ ==  '2' ]]; then Processor=$CPU_Anyi686Compatible
  elif [[ $__cpu__ ==  '3' ]]; then
    DialogMenuBox "Supported CPUs: Intel's CPUs"          \
      0 "Pentium                                        " \
      1 "Pentium MMX                                    " \
      2 "Pentium Pro                                    " \
      3 "Pentium II                                     " \
      4 "Pentium II Celeron                             " \
      5 "Pentium II Xeon                                " \
      6 "Pentium III                                    " \
      7 "Pentium III Celeron                            " \
      8 "Pentium III Xeon                               " \
      9 "Pentium M                                      " \
      A "Pentium M Celeron                              " \
      B "Pentium 4                                      " \
      C "Pentium 4 Celeron                              " \
      D "Pentium 4 Xeon                                 " \
      E "Pentium 4 Prescott                             " \
      F "Pentium 4 Celeron D                            " \
      G "Pentium 4 Nocona                               " \
      H "Pentium D                                      " \
      I "Core Solo                                      " \
      J "Core Duo                                       " \
      K "Core 2 Duo                                     "
    DialogGetAns __cpu__
      if [[ $__cpu__ ==  '0' ]]; then Processor=$CPU_IntelPentium
    elif [[ $__cpu__ ==  '1' ]]; then Processor=$CPU_IntelPentiumMMX
    elif [[ $__cpu__ ==  '2' ]]; then Processor=$CPU_IntelPentiumPro
    elif [[ $__cpu__ ==  '3' ]]; then Processor=$CPU_IntelPentiumII
    elif [[ $__cpu__ ==  '4' ]]; then Processor=$CPU_IntelCeleron
    elif [[ $__cpu__ ==  '5' ]]; then Processor=$CPU_IntelP2Xeon
    elif [[ $__cpu__ ==  '6' ]]; then Processor=$CPU_IntelPentiumIII;  UseSSEMOp=1
    elif [[ $__cpu__ ==  '7' ]]; then Processor=$CPU_IntelCeleron2;    UseSSEMOp=1
    elif [[ $__cpu__ ==  '8' ]]; then Processor=$CPU_IntelP3Xeon;      UseSSEMOp=1
    elif [[ $__cpu__ ==  '9' ]]; then Processor=$CPU_IntelPentiumM;    UseSSEMOp=1
    elif [[ $__cpu__ == [Aa] ]]; then Processor=$CPU_IntelCeleronM;    UseSSEMOp=1
    elif [[ $__cpu__ == [Bb] ]]; then Processor=$CPU_IntelPentium4;    UseSSEMOp=1
    elif [[ $__cpu__ == [Cc] ]]; then Processor=$CPU_IntelCeleron3;    UseSSEMOp=1
    elif [[ $__cpu__ == [Dd] ]]; then Processor=$CPU_IntelP4Xeon;      UseSSEMOp=1
    elif [[ $__cpu__ == [Ee] ]]; then Processor=$CPU_IntelPrescott;    UseSSEMOp=1
    elif [[ $__cpu__ == [Ff] ]]; then Processor=$CPU_IntelCeleronD;    UseSSEMOp=1
    elif [[ $__cpu__ == [Gg] ]]; then Processor=$CPU_IntelNocona;      UseSSEMOp=1
    elif [[ $__cpu__ == [Hh] ]]; then Processor=$CPU_IntelPentiumD;    UseSSEMOp=1
    elif [[ $__cpu__ == [Ii] ]]; then Processor=$CPU_IntelCoreSolo;    UseSSEMOp=1
    elif [[ $__cpu__ == [Jj] ]]; then Processor=$CPU_IntelCoreDuo;     UseSSEMOp=1
    elif [[ $__cpu__ == [Kk] ]]; then Processor=$CPU_IntelCore2Duo;    UseSSEMOp=1
    else                              Processor=$CPU_Anyi586Compatible
    fi
  elif [[ $__cpu__ ==  '4' ]]; then
    DialogMenuBox "Supported CPUs: AMD CPUs"              \
      0 "K6                                             " \
      1 "K6 II                                          " \
      2 "K6 III                                         " \
      3 "Athlon                                         " \
      4 "Athlon Thunderbird                             " \
      5 "Duron                                          " \
      6 "Athlon 4                                       " \
      7 "Athlon XP                                      " \
      8 "Athlon MP                                      " \
      9 "K8                                             " \
      A "K8 Opteron                                     " \
      B "Athlon 64                                      " \
      C "Athlon 64 Sempron                              " \
      D "Athlon 64 Turion                               " \
      E "Athlon 64 FX                                   " \
      F "Athlon 64 X2                                   "
    DialogGetAns __cpu__
      if [[ $__cpu__ ==  '0' ]]; then Processor=$CPU_AMDK6
    elif [[ $__cpu__ ==  '1' ]]; then Processor=$CPU_AMDK6II
    elif [[ $__cpu__ ==  '2' ]]; then Processor=$CPU_AMDK6III
    elif [[ $__cpu__ ==  '3' ]]; then Processor=$CPU_AMDAthlon
    elif [[ $__cpu__ ==  '4' ]]; then Processor=$CPU_AMDAthlonTBird
    elif [[ $__cpu__ ==  '5' ]]; then Processor=$CPU_AMDDuron
    elif [[ $__cpu__ ==  '6' ]]; then Processor=$CPU_AMDAthlon4;       UseSSEMOp=1
    elif [[ $__cpu__ ==  '7' ]]; then Processor=$CPU_AMDAthlonXP;      UseSSEMOp=1
    elif [[ $__cpu__ ==  '8' ]]; then Processor=$CPU_AMDAthlonMP;      UseSSEMOp=1
    elif [[ $__cpu__ ==  '9' ]]; then Processor=$CPU_AMDK8;            UseSSEMOp=1
    elif [[ $__cpu__ == [Aa] ]]; then Processor=$CPU_AMDOpteron;       UseSSEMOp=1
    elif [[ $__cpu__ == [Bb] ]]; then Processor=$CPU_AMDAthlon64;      UseSSEMOp=1
    elif [[ $__cpu__ == [Cc] ]]; then Processor=$CPU_AMDSempron;       UseSSEMOp=1
    elif [[ $__cpu__ == [Dd] ]]; then Processor=$CPU_AMDTurion;        UseSSEMOp=1
    elif [[ $__cpu__ == [Ee] ]]; then Processor=$CPU_AMDAthlonFX;      UseSSEMOp=1
    elif [[ $__cpu__ == [Ff] ]]; then Processor=$CPU_AMDAthlonX2;      UseSSEMOp=1
    else                              Processor=$CPU_Anyi586Compatible
    fi
  else
    DialogMenuBox "Supported CPUs: Other Vendors' CPUs"   \
      0 "IDT Winchip C6                                 " \
      1 "IDT Winchip 2                                  " \
      2 "Via C3                                         " \
      3 "Via C3-2                                       "
    DialogGetAns __cpu__
      if [[ $__cpu__ == '0' ]]; then Processor=$CPU_IDTWinChipC6;
    elif [[ $__cpu__ == '1' ]]; then Processor=$CPU_IDTWinChip2;
    elif [[ $__cpu__ == '2' ]]; then Processor=$CPU_ViaC3;
    elif [[ $__cpu__ == '3' ]]; then Processor=$CPU_ViaC3II;           UseSSEMOp=1
    else                             Processor=$CPU_Anyi586Compatible
    fi
  fi

  local __ams__="Autodetecting MMX, 3DNow! and SSE will override the default per CPU's settings (not reccommended if for building generic binary distribution package)."
  local __adt__

  $DlgCmd --backtitle "$DTitle"                                  \
          --begin 3 3 --infobox                                  \
              "$__ams__" 5 74                                    \
          --and-widget --menu                                    \
             "MMX, 3DNow! and SSE Extensions" 9 $DWidth 2        \
             0 "Autodetect                                     " \
             1 "Do not autodetect                              " 2> $DAnsFl
  CheckCancel
  DialogGetAns __adt__
    if [[ $__adt__ ==  '0' ]]; then DetectPEx='Y'
  else                              DetectPEx='N'
  fi

  local __chk__
  DialogCheckBox "Options                         "     \
    0 "Use color for VortexGE console messages    " on  \
    1 "Use xf86vm extension                       " on  \
    2 "Use X shared memory (xshm) extension       " on  \
    3 "Enable PNG file loading support            " on  \
    4 "Enable JPEG file loading support           " on  \
    5 "Enable TIFF file loading support           " on  \
    6 "Enable GIF file loading support            " on  \
    7 "Enable inline assembly optimizations       " on  \
    8 "Enable debugging information               " off \
    9 "Build the static library also              " off
  DialogGetAns __chk__
  if [[ `echo "$__chk__" | grep '0'` != '' ]]; then ColorMsgs='Y'; else ColorMsgs='N'; fi
  if [[ `echo "$__chk__" | grep '1'` != '' ]]; then UseXF86VM='Y'; else UseXF86VM='N'; fi
  if [[ `echo "$__chk__" | grep '2'` != '' ]]; then UseXShmEx='Y'; else UseXShmEx='N'; fi
  if [[ `echo "$__chk__" | grep '3'` != '' ]]; then UseLibPng='Y'; else UseLibPng='N'; fi
  if [[ `echo "$__chk__" | grep '4'` != '' ]]; then UseLibJpg='Y'; else UseLibJpg='N'; fi
  if [[ `echo "$__chk__" | grep '5'` != '' ]]; then UseLibTif='Y'; else UseLibTif='N'; fi
  if [[ `echo "$__chk__" | grep '6'` != '' ]]; then UseLibGif='Y'; else UseLibGif='N'; fi
  if [[ `echo "$__chk__" | grep '7'` != '' ]]; then InlineAsm='Y'; else InlineAsm='N'; fi
  if [[ `echo "$__chk__" | grep '8'` != '' ]]; then UseDebugI='Y'; else UseDebugI='N'; fi
  if [[ `echo "$__chk__" | grep '9'` != '' ]]; then BuildALib='Y'; else BuildALib='N'; fi

  ##### Enable these options by default
  AlignedMO='Y'
  QuadWrdMO='Y'

  ##### Ask for optimization level
  if [[ $UseDebugI == 'Y' ]]; then
    DialogRadioBox "Optimization level              "     \
      0 "Disable optimization (-O0)                 " on  \
      1 "Level 1              (-O1)                 " off \
      2 "Level 2              (-O2)                 " off \
      3 "Level 3              (-O3)                 " off
  else
    DialogRadioBox "Optimization level              "     \
      0 "Disable optimization (-O0)                 " off \
      1 "Level 1              (-O1)                 " off \
      2 "Level 2              (-O2)                 " off \
      3 "Level 3              (-O3)                 " on
  fi
  DialogGetAns OptzLevel

  ##### Remove the temporary answer file
  rm -f $DAnsFl
}
