// ############################################################################################
//  VLGame-AppClass.h
//  A demo application for VortexGE (definition of VLGame's application class)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _DEMO___VLGAME_APP_CLASS_H_
#define _DEMO___VLGAME_APP_CLASS_H_

//#define __HIGH_RESOLUTION__

// VLGame's headers
#include "VLGame-AThread.h"
#include "VLGame-Entities.h"


//
// VLGame's application class
//
typedef CBasicApplication<> VLGApplicationBase;

class VLGApplication : public VLGApplicationBase {
  public:
    VLGApplication();
    ~VLGApplication() throw();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventKeyRelease(EKeySymbol eKeySym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventWindowActivated();
    virtual void EventWindowDeactivated();
    virtual void EventCloseWindow();

  private:
    // Helper functions
    CDataSource &_GPF(const string &fileName);
    void _initMediaPackage();
    void _initScript();
    void _loadAudios();
    void _loadImages();
    void _createScene();
    void _destroyScene();

    void _testScript();

    // Constants
    static const char *MediaPackageMagicNumbers;
    static const char *MediaPackageFileName;
    static const char *SaveGameFileName;
    static const char *VortexLuaWorkingDirectory;

    // Core data
    VLGAudioThread     audioThread;
    CLuaManager        luaManager;
    CArchiveDataSource mediaPackage;
    CMediaPool      mediaPool;
    VLGKeyMap          keyMap;

    // Entities
    VLGPlayer               player;
    VLGUnitScene_StartRoom *uscStartRoom;

    // Sound buffers
    CSoundBuffer *sbWalk, *sbDoor, *sbTalk, *sbFire;
};

#endif
