// ############################################################################################
//  tut01c.cc
//  A tutorial application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// The VortexGE include file
#include <VMain.h>

// Specialized event handler class
//
// NOTE: we derive it from CEventHandlerExt because we want FPS control
class MyEventHandler : public CEventHandlerExt {
  private:
    // Variables
    CVideoEnv  _csVideoEnv;
    C3DEnv     _cs3DEnv;
    CAudioEnv &_csAudioEnv;

  public:
    // Constructor
    MyEventHandler();
    // Reimplement only the needed event handlers
    virtual bool EventMainLoopEnter();
    virtual void EventButtonRelease(EBtnSymbol eButtonSym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
};


// Main program
int main()
{
  // Local variables
  MyEventHandler csMyEH;

  // Invoke the default main loop
  int32_t n32AvgFPS = csMyEH.DoMainLoop();
  if(n32AvgFPS < 0) return(-1);

  // Exiting
  csMyEH.HookedVideoEnv()->MessageBox("End Program", "Click \"OK\" to exit ...");
  VInfo(_N_, _F_, _L_, "Program ended, average frame rate = %.2f FPS",
        float(n32AvgFPS) / 100.0f);

  // OK !
  return(0);
}


// MyEventHandler implementation
MyEventHandler::MyEventHandler()
: CEventHandlerExt(&_csVideoEnv), _csAudioEnv(CAudioEnv::AutoDetect())
{
}

bool MyEventHandler::EventMainLoopEnter()
{
  // Initialize system
  _csAudioEnv.DoQuickInit();
  _csVideoEnv.DoQuickInit("Tutorial 1c : Using the CEventHandlerExt Class", 640, 480);
  _cs3DEnv.Initialize(&_csVideoEnv);
  _csVideoEnv.SetForegroundColor(_csVideoEnv.CreateColor(255, 192, 255));
  return(true);
}

void MyEventHandler::EventButtonRelease(EBtnSymbol eButtonSym)
{
  if(eButtonSym == VBS_MB1) EventCloseWindow();
}

void MyEventHandler::EventUpdateFrame(bool bWindowIsActive)
{
  // Update everything if the window is active (has focus)
  if(bWindowIsActive) {
    // Update audio
    _csAudioEnv.UpdateBuffer();
    // Update video
    _cs3DEnv.ClearRenderBuffer(V_RGB(12, 12, 16));
    _cs3DEnv.ClearZBuffer();
    _csVideoEnv.PutRenderImageOnDisplay();
    _csVideoEnv.DrawText(10, 20, GetFPSInfoString());
    _csVideoEnv.DrawText(10, 50, "Click the left mouse button to exit ...");
    _csVideoEnv.UpdateDisplay();
  }
  // Update everything if the window is inactive
  else {
    _csVideoEnv.PutRenderImageOnDisplay();
    _csVideoEnv.UseDefaultBigFont();
    _csVideoEnv.DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    _csVideoEnv.UseDefaultSmallFont();
    _csVideoEnv.UpdateDisplay();
  }
}

