/** \file AALSAEnvironment.h */
// ############################################################################################
//  AALSAEnvironment.h
//  Definition & implementation of CAlsaAudioEnv class using ALSA audio system
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#include "../../VConfig.h"
#if !defined(_AUDIO_ENVIRONMENT___PCM_ALSA___AALSA_ENVIRONMENT_H_) && defined(__USE_ALSA_AUDIO__)
#define _AUDIO_ENVIRONMENT___PCM_ALSA___AALSA_ENVIRONMENT_H_

// VortexGE headers
#include "../NonDummyAudioEnvBaseClass.h"


// Foward reference from "alsa/asoundlib.h"
// NOTE: If these lines not compile, use these steps to fix :
//         1. Check inside "asoundlib.h" and find the real structure names which
//            are used for typedef-ing "snd_pcm_t" and "snd_pcm_hw_params_t".
//         2. Replace the lines below with the found structure names.
struct _snd_pcm;
struct _snd_pcm_hw_params;


namespace vge {

  // Foward reference from "lib/AudioEnvironment/PCM_ALSA/AALSASoundBuffer.h"
  class CAlsaSoundBuffer;


  //!
  //! \brief ALSA audio environment class
  //!
  //! \api pretty stable
  class VGE_EXPORT CAlsaAudioEnv : public CAudioEnvCImp, VCLASS_SINGLETON {
    // Support for singleton class
    VCLASS_SINGLETON_DECLARE(CAlsaAudioEnv)

    public:
      CAlsaAudioEnv();
      virtual ~CAlsaAudioEnv() throw();
      // Quick initialization functions
      virtual void DoQuickInit(const char *pcDeviceName = NULL);
      // Initialization functions
      virtual size_t QueryDeviceName(char *pcResult, size_t bufferSize = PATH_MAX);
      virtual void OpenDevice(const char *pcDeviceName);
      virtual void InitializeDevice(const TAudioSettings *ptPreferred = NULL, TAudioSettings *ptRealized = NULL);
      virtual void ActivateDevice();
      virtual void DeactivateDevice();
      virtual void GetAudioSpec(uint32_t *un32Channels, uint32_t *un32Bits, uint32_t *un32Rates, ESoundBufferFormat *eFormat, uint32_t *un32FragSize) const;
      virtual void SetMidiPlayer(CMidiPlayer *pcsPlayer);
      // Sound functions
      virtual bool IsAvailable() const { return(_tD.deviceHandle); }
      virtual void SuspendDevice()     { _csTWriter.Suspend(); }
      virtual void ResumeDevice()      { _csTWriter.Resume(); }
      virtual void UpdateBuffer()      { if(_tD.deviceHandle) _pfnDoSoundMixing(); }
      // Sound buffer functions
      virtual CSoundBuffer *NewSoundBuffer();

      // Friend classes
      friend class CAlsaSoundBuffer;

    protected:
      // Initialization functions
      bool _AdjustDeviceName_(char *pcDeviceName);
      // Sound buffer functions
      void _ClearBuffer_(u_char *pucBuff, uint32_t un32BuffSize, uint8_t un8Sign, uint8_t un8Bits);
      // Thread functions
      static uint64_t _ThreadProc(TThreadData *ptThreadData);

      // Data
      struct _TDSPData : public _TCommonDSPData {
        char               acDevName[PATH_MAX]; // Audio device path
        _snd_pcm           *deviceHandle;       // ALSA device handle
        _snd_pcm_hw_params *hwParams;           // ALSA hardware parameters
      } _tD;

      // Pointer to the one and the only CAlsaAudioEnv instance
      static CAlsaAudioEnv *_instance;
  };

} // namespace vge
#endif
