/** \file Entities.h */
// ############################################################################################
//  Entities.h
//  Definition & implementation of various game entity classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _GAME_OBJECT___ENTITIES_H_
#define _GAME_OBJECT___ENTITIES_H_

// VortexGE headers
#include "../3DEnvironment/Sprite2D.h"
#include "BasicEntities.h"

//
// Convenience macros for initializing game entity classes classes using
// initialization structs from their derived classes
//
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CGenttyBase base class.
//!
//! \api preliminary experiment
#define VIS_CGenttyBase(SINIT)             CGenttyBase            (SINIT.entityName)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CGenttyBaseXSpace base class.
//!
//! \api preliminary experiment
#define VIS_CGenttyBaseXSpace(SINIT)       CGenttyBaseXSpace      (SINIT.entityName, SINIT.xspaceName, SINIT.xspaceGroupName, SINIT.scriptManager)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CGenttyBaseEBC base class.
//!
//! \api preliminary experiment
#define VIS_CGenttyBaseEBC(SINIT)          CGenttyBaseEBC         (SINIT.entityName, SINIT.exeName, SINIT.scriptManager)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CGenttyBaseXSpaceEBC base class.
//!
//! \api preliminary experiment
#define VIS_CGenttyBaseXSpaceEBC(SINIT)    CGenttyBaseXSpaceEBC   (SINIT.entityName, SINIT.xspaceName, SINIT.xspaceGroupName, SINIT.exeName, SINIT.scriptManager)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CGenttyBaseAImage base class.
//!
//! \api preliminary experiment
#define VIS_CGenttyBaseAImage(SINIT)       CGenttyBaseAImage      (SINIT.entityName, SINIT.aimage, SINIT.aimageASInit)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CBasicCharacter base class.
//!
//! \api preliminary experiment
#define VIS_CBasicCharacter(SINIT)         CBasicCharacter        (SINIT)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this macro in the ctor of all derived classes to initialize the vge::CBasicCharacterSprite2D cbase lass.
//!
//! \api preliminary experiment
#define VIS_CBasicCharacterSprite2D(SINIT) CBasicCharacterSprite2D(SINIT)

//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this compound macro in the ctor of all classes derived from vge::CGenttyBaseXSpaceEBC to initialize all the base classes.
//!
//! \api preliminary experiment
#define VIC_CGenttyBaseXSpaceEBC(SINIT)  \
  VIS_CGenttyBase               (SINIT), \
  VIS_CGenttyBaseXSpace         (SINIT), \
  VIS_CGenttyBaseEBC            (SINIT), \
  VIS_CGenttyBaseXSpaceEBC      (SINIT)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this compound macro in the ctor of all classes derived from vge::CBasicCharacter to initialize all the base classes.
//!
//! \api preliminary experiment
#define VIC_CBasicCharacter(SINIT)  \
  VIC_CGenttyBaseXSpaceEBC (SINIT), \
  VIS_CGenttyBaseAImage    (SINIT), \
  VIS_CBasicCharacter      (SINIT)
//!
//! \ingroup Game_Entity_Macros
//! \hideinitializer
//! Call this compound macro in the ctor of all classes derived from vge::CBasicCharacterSprite2D to initialize all the base classes.
//!
//! \api preliminary experiment
#define VIC_CBasicCharacterSprite2D(SINIT)  \
  VIC_CBasicCharacter              (SINIT), \
  VIS_CBasicCharacterSprite2D      (SINIT)


namespace vge {

  //!
  //! \ingroup Game_Entity_IStruct
  //! \brief Initialization structure used for initializing CBasicCharacter class instance
  //!
  //! \api preliminary experiment
  struct VGE_EXPORT TInit_BasicChar {
    // Redefine the data type for convenience
    typedef CGenttyBaseAImage::CInit_AnimSet CInit_AnimSet;

    // Redefine the constant for convenience
    static const CInit_AnimSet *&CreateEmptyAnimationSet;

    // Data
    const char          *entityName;      //!< Name of the entity.
    const char          *xspaceName;      //!< Name of the entity's CLuaExecutionSpace instance.
    const char          *xspaceGroupName; //!< Group name of the entity's CLuaExecutionSpace instance.
    const char          *exeName;         //!< Name of the entity's CLuaExecutable instance.
    CLuaManager         *scriptManager;   //!< Script manager to be used with the entity.
    CAnimatedImage_MP    aimage;          //!< Pooled CAnimatedImage instance to be used with the entity.
    const CInit_AnimSet *aimageASInit;    //!< Advanced frame controller to be used with the animated image.

    // Ctor for convenience
    inline TInit_BasicChar(const char              *entityName_      = NULL,
                           const char              *xspaceName_      = NULL,
                           const char              *xspaceGroupName_ = NULL,
                           const char              *exeName_         = NULL,
                           CLuaManager             *scriptManager_   = NULL,
                           const CAnimatedImage_MP &aimage_          = CAnimatedImage_MP(),
                           const CInit_AnimSet     *aimageASInit_    = NULL)
    : entityName     (entityName_),
      xspaceName     (xspaceName_),
      xspaceGroupName(xspaceGroupName_),
      exeName        (exeName_),
      scriptManager  (scriptManager_),
      aimage         (aimage_),
      aimageASInit   (aimageASInit_)
    {}
  };

  //!
  //! \brief The base of all playable (PC) and non playable character (NPC) classes
  //!
  //! All classes derived from this CBasicCharacter class must also initialize
  //! the virtually derived CGenttyBase, CGenttyBaseXSpace and CGenttyBaseEBC,
  //! CGenttyBaseXSpaceEBC and CGenttyBaseAImage classes from within the derived
  //! class ctor.
  //! \n\n
  //! See the page \ref Game_Entity_Macros for a more convenient way when
  //! initializing using \ref Game_Entity_IStruct.
  //!
  //! \api preliminary experiment
  class VGE_EXPORT CBasicCharacter : virtual public CGenttyBaseXSpaceEBC,
                                     virtual public CGenttyBaseAImage {
    public:
      //!
      //! Construct a new basic character class using the settings
      //! described in the sturcture
      CBasicCharacter(const TInit_BasicChar &init);
      //!
      //! Standard dtor
      virtual ~CBasicCharacter() throw();
  };


  //!
  //! \ingroup Game_Entity_IStruct
  //! \brief Initialization structure used for initializing \ref CSprite2D's advanced properties
  //!
  //! \api preliminary experiment
  struct TInit_Spr2DAdv {
    EPivot         pivot;             //!< Pivot position of the sprite's image.
    uint32_t       excludedColorVRGB; //!< Image's color to be excluded when rendering (to be assumed as transparent area).
    TAmbientLight *ambientLight;      //!< Custom ambient light to be used with the sprite.
    TOmniLight    *omniLight;         //!< Custom omni light to be used with the sprite.
    float          collisionRadius;   //!< Collision radius of the sprite.

    TInit_Spr2DAdv(EPivot         pivot_             = EP_BottomCenter,
                   uint32_t       excludedColorVRGB_ = EXCLUDED_COLOR_FIRST_PIXEL,
                   TAmbientLight *ambientLight_      = NULL,
                   TOmniLight    *omniLight_         = NULL,
                   float          collisionRadius_   = -1.0f)
    : pivot            (pivot_),
      excludedColorVRGB(excludedColorVRGB_),
      ambientLight     (ambientLight_),
      omniLight        (omniLight_),
      collisionRadius  (collisionRadius_)
    {}
  };

  //!
  //! \ingroup Game_Entity_IStruct
  //! \brief Structure used for initializing CBasicCharacterSprite2D class instance
  //!
  //! \api preliminary experiment
  struct VGE_EXPORT TInit_BasicCharSpr2D : public TInit_BasicChar {
    // Pull the symbols for convenience
    using TInit_BasicChar::CInit_AnimSet;
    using TInit_BasicChar::CreateEmptyAnimationSet;

    // Data
    CAnimatedImage_MP        spriteAImage;  //!< Pooled CAnimatedImage instance to be used by the sprite.
    const CInit_AnimSet     *spriteASInit;  //!< Advanced frame controller to be used with the sprite's animated image.
    const vector<TFloatXYZ> *spriteInitPos; //!< Initial position of the sprite.
    TInit_Spr2DAdv           spriteInitAdv; //!< Advanced sprite intialization parameters.

    // Ctor for convenience
    inline TInit_BasicCharSpr2D(const char              *entityName_      = NULL,
                                const char              *xspaceName_      = NULL,
                                const char              *xspaceGroupName_ = NULL,
                                const char              *exeName_         = NULL,
                                CLuaManager             *scriptManager_   = NULL,
                                const CAnimatedImage_MP &aimage_          = CAnimatedImage_MP(),
                                const CInit_AnimSet     *aimageASInit_    = NULL,
                                const CAnimatedImage_MP &spriteAImage_    = CAnimatedImage_MP(),
                                const CInit_AnimSet     *spriteASInit_    = NULL,
                                const vector<TFloatXYZ> *spriteInitPos_   = NULL,
                                const TInit_Spr2DAdv    &spriteInitAdv_   = TInit_Spr2DAdv())
    : TInit_BasicChar(entityName_, xspaceName_, xspaceGroupName_, exeName_, scriptManager_, aimage_, aimageASInit_),
      spriteAImage (spriteAImage_),
      spriteASInit (spriteASInit_),
      spriteInitPos(spriteInitPos_),
      spriteInitAdv(spriteInitAdv_)
    {}
  };

  //!
  //! \brief The base of all playable (PC) and non playable character (NPC) classes
  //!        with Sprite2D
  //!
  //! All classes derived from this CBasicCharacter class must also initialize
  //! the virtually derived CGenttyBase, CGenttyBaseXSpace and CGenttyBaseEBC,
  //! CGenttyBaseXSpaceEBC, CGenttyBaseAImage and CBasicCharacter classes from 
  //! within the derived class ctor.
  //! \n\n
  //! See the page \ref Game_Entity_Macros for a more convenient way when
  //! initializing using \ref Game_Entity_IStruct.
  //!
  //! \api preliminary experiment
  class VGE_EXPORT CBasicCharacterSprite2D : virtual public CBasicCharacter {
    public:
      //!
      //! Construct a new basic character class using the settings
      //! described in the sturcture
      CBasicCharacterSprite2D(const TInit_BasicCharSpr2D &init);
      //!
      //! Standard dtor
      virtual ~CBasicCharacterSprite2D() throw();

      // Access to the CSprite2D instance
      //!
      //! Direct access to the CSprite2D instance of this entity.
      inline CSprite2D &iSprite2D()
      { return(_sprite); }
      //!
      //! Direct access to the CSprite2D instance of this entity.
      inline const CSprite2D &iSprite2D() const
      { return(_sprite); }
      //!
      //! Reinitialize the CSprite2D instance
      void ReinitializeSprite2D(const CAnimatedImage_MP &spriteAImage_  = CAnimatedImage_MP(),
                                const CInit_AnimSet     *spriteASInit_  = NULL,
                                const vector<TFloatXYZ> *spriteInitPos_ = NULL,
                                const TInit_Spr2DAdv    &spriteInitAdv_ = TInit_Spr2DAdv());
      //!
      //! Reinitialize the CSprite2D instance
      void ReinitializeSprite2D(const CAnimatedImage_MP &spriteAImage_  = CAnimatedImage_MP(),
                                const CInit_AnimSet     *spriteASInit_  = NULL,
                                const TFloatXYZ         *spriteInitPos_ = NULL,
                                const TInit_Spr2DAdv    &spriteInitAdv_ = TInit_Spr2DAdv());

      // Access to the sprite's CAnimatedImage instance
      //!
      //! Direct access to the sprite's CAnimatedImage instance of this entity.
      inline CAnimatedImage *iSpriteAImage()
      { return(_saimage.MediaInst()); }
      //!
      //! Direct access to the sprite's CAnimatedImage instance of this entity.
      inline const CAnimatedImage *iSpriteAImage() const
      { return(_saimage.MediaInst()); }

      // Access to the sprite's CAImgControl instance
      //!
      //! Direct access to the sprite's CAImgControl instance of this entity.
      inline CAImgControl *iSpriteAImgControl()
      { return(_saictl); }
      //!
      //! Direct access to the sprite's CAImgControl instance of this entity.
      inline const CAImgControl *iSpriteAImgControl() const
      { return(_saictl); }

    protected:
      CSprite2D          _sprite;  // The sprite
      CAnimatedImage_MP  _saimage; // Pooled CAnimatedImage instance to be used when drawing sprite
      CAImgControl      *_saictl;  // Advanced frame control for the animated sprite image
  };

} // namespace vge
#endif
