// ############################################################################################
//  fontDisplay.cc
//  Fonts displayer
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#include "../lib/VConfig.h"
#ifndef __USE_X11_VIDEO__

// Unix standard headers
#include <stdio.h>

int main()
{
  printf("\nfontDisplay is not usable since VortexGE was not built using the X Window system.\n\n");
  return(0);
}

#else

// Unix standard headers
#include <stdint.h>
#include <stdio.h>

// X11 headers
#include <X11/Xlib.h>
#include <X11/Xutil.h>


// These characters are actually ASCII codes, if you open this file with Anjuta or KEdit you
// will see them correctly
//   ⅓⅔⅛⅜⅝⅞←↑→↓↔↕↨▪▫▲►▼◄○●☺☻☼♀♂♠♣♥♦♪♫
//   ⅓ÇüéâäàåçêëèïîìÄÅÉæÆôöòûùÿÖÜ¢£¥₧ƒáíóúñÑªº¿⌐¬½¼¡«»░▒▓│┤╡╢╖╕╣║╗╝╜╛┐└┴┬├─┼╞╟╚╔╩╦╠═╬╧╨╤╥╙╘╒╓╫╪┘┌█▄▌▐▀αßΓπΣσµτΦΘΩδ∞φε∩≡±≥≤⌠⌡÷≈³¹†√ⁿ²■₪₫∆⌂
// These are Japanese hiragana and katakana
//   ぁあぃいぅうぇえぉおかがきぎくぐけげこごさざしじすずせぜそぞただちぢっつづてでとどなにぬねのはばぱひびぴふぶぷへべぺほぼぽまみむめもゃやゅゆょよらりるれろゎわゐゑをんゔ゛゜ゝゞ
//   ァアィイゥウェエォオカガキギクグケゲコゴサザシジスズセゼソゾタダチヂッツヅテデトドナニヌネノハバパヒビピフブプヘベペホボポマミムメモャヤュユョヨラリルレロヮワヰヱヲンヴヵヶヷヸヹヺ・ーヽヾ
//   ｡｢｣､･ｦｧｨｩｪｫｬｭｮｯｰｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓﾔﾕﾖﾗﾘﾙﾚﾛﾜﾝﾞﾟ

// Macros
// WARNING: Modifying these means modifying also the _FONT_ROW_ and _FONT_COL_ values in
//          "lib/2DManipulation/BitmapFont.h".
#define _XLOOPS_ 48
#define _YLOOPS_ 6


// Get XCharStruct
XCharStruct *CVT(XFontStruct *ptFont, uint32_t un32Char)
{
  // Temporary variables
  uint32_t un32Rows, un32Pages;
  uint32_t un32Byte1, un32Byte2;
  bool     bValid;

  // Check for validity
  bValid    = true;
  un32Rows  = ptFont->max_byte1 - ptFont->min_byte1 + 1;
  un32Pages = ptFont->max_char_or_byte2 - ptFont->min_char_or_byte2 + 1;
  if(un32Rows == 1) {
    un32Byte1 = 0;
    un32Byte2 = 0;
    if((ptFont->min_char_or_byte2 > un32Char) || (ptFont->max_char_or_byte2 < un32Char)) bValid = false;
  }
  else {
    un32Byte1 = un32Char >> 8;
    un32Byte2 = un32Char & 0xFF;
    if((ptFont->min_char_or_byte2 > un32Byte2) || (ptFont->max_char_or_byte2 < un32Byte2) ||
      (ptFont->min_byte1 > un32Byte1) || (ptFont->max_byte1 < un32Byte1)) bValid = false;
  }

  // Return the char
  if(bValid) {
    if(ptFont->per_char) {
      if(un32Rows == 1)
        return(ptFont->per_char + (un32Char - ptFont->min_char_or_byte2));
      else
        return(ptFont->per_char + ((un32Byte1 - ptFont->min_byte1) * un32Pages) + (un32Byte2 - ptFont->min_char_or_byte2));
    }
    else
      return(&ptFont->min_bounds);
  }
  return(NULL);
}

// Draw fonts
void DrawFont(const char *pcFont, const char *pcReg, uint8_t un8Size, uint8_t un8Page)
{
  // Temporary variables
  Display       *ptDisp;
  int32_t       n32Screen;
  int32_t       n32Depth;
  XFontStruct   *ptFont[4];
  Window        tWin;
  char          acTitle[256], acFont[256];
  XSizeHints    tSize;
  GC            tGC;
  Pixmap        tPixmap;
  XEvent        tEvent;
  int32_t       n32FW, n32FH, n32FA, n32FT;
  int32_t       n32W, n32H, n32F, n32Y, n32X, n32Pos;
  bool          bExit;
  XChar2b       wcP;

  // Open display
  ptDisp    = XOpenDisplay(NULL);
  n32Screen = DefaultScreen(ptDisp);
  n32Depth  = DefaultDepth(ptDisp, n32Screen);
  if(n32Depth == 8) {
    printf("Must use 16 or 24 bpp !");
    return;
  }

  // Create 1st font
  snprintf(acFont, sizeof(acFont), "*-%s-medium-r-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
  ptFont[0] = XLoadQueryFont(ptDisp, acFont);
  if(ptFont[0] == NULL) printf("Failed : %s\n", acFont);

  // Create 2nd font
  snprintf(acFont, sizeof(acFont), "*-%s-medium-i-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
  ptFont[1] = XLoadQueryFont(ptDisp, acFont);
  if(ptFont[1] == NULL) {
    printf("Failed : %s\n", acFont);
    snprintf(acFont, sizeof(acFont), "*-%s-medium-o-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
    ptFont[1] = XLoadQueryFont(ptDisp, acFont);
    if(ptFont[1] == NULL) printf("Failed : %s\n", acFont);
  }

  // Create 3rd font
  snprintf(acFont, sizeof(acFont), "*-%s-bold-r-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
  ptFont[2] = XLoadQueryFont(ptDisp, acFont);
  if(ptFont[2] == NULL) printf("Failed : %s\n", acFont);

  // Create 4th font
  snprintf(acFont, sizeof(acFont), "*-%s-bold-i-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
  ptFont[3] = XLoadQueryFont(ptDisp, acFont);
  if(ptFont[3] == NULL) {
    printf("Failed : %s\n", acFont);
    snprintf(acFont, sizeof(acFont), "*-%s-bold-o-*-*-%u-*-*-*-*-*-%s-*", pcFont, un8Size, pcReg);
    ptFont[3] = XLoadQueryFont(ptDisp, acFont);
    if(ptFont[3] == NULL) printf("Failed : %s\n", acFont);
  }

  // Check
  if((ptFont[0] == NULL) && (ptFont[1] == NULL) && (ptFont[2] == NULL) && (ptFont[3] == NULL)) return;

  // Calculate font size
  n32FW = n32FH = n32FA = 0;
  for(n32F = 0; n32F < 4; n32F++) {
    if(ptFont[n32F]) {
      n32FT = ptFont[n32F]->max_bounds.width;
      if(n32FT > n32FW) n32FW = n32FT;
      n32FT = ptFont[n32F]->ascent + ptFont[n32F]->descent;
      if(n32FT > n32FH) n32FH = n32FT;
      n32FT = ptFont[n32F]->ascent;
      if(n32FT > n32FA) n32FA = n32FT;
    }
  }
  n32FW += 4; n32FH += 4;
  n32W = n32FW * _XLOOPS_;
  n32H = n32FH * _YLOOPS_ * 4;
  if(n32W <= 0) n32W = n32FW;
  if(n32H <= 0) n32H = n32FH;

  // Create window
  snprintf(acTitle, sizeof(acTitle),
           "%s, %s, page %u, %upt ", pcFont, pcReg, un8Page, un8Size);
  tWin = XCreateSimpleWindow(ptDisp, RootWindow(ptDisp, n32Screen),
                             0, 0, n32W, n32H,
                             4, WhitePixel(ptDisp, n32Screen),
                             BlackPixel(ptDisp, n32Screen));
  tSize.flags      = PMinSize | PMaxSize;
  tSize.min_width  = tSize.max_width  = n32W;
  tSize.min_height = tSize.max_height = n32H;
  XSetStandardProperties(ptDisp, tWin,
                         acTitle, acTitle, None, NULL, 0, &tSize);
  XSelectInput(ptDisp, tWin, ButtonPressMask | ExposureMask);
  XMapWindow(ptDisp, tWin);
  XMoveWindow(ptDisp, tWin, 10, 10);

  // Create GC
  tGC = XCreateGC(ptDisp, RootWindow(ptDisp, n32Screen), 0, NULL);

  // Create pixmap
  tPixmap = XCreatePixmap(ptDisp, tWin, n32W, n32H, n32Depth);

  // Print characters
  XSetForeground(ptDisp, tGC, 0x00000000);
  XFillRectangle(ptDisp, tPixmap, tGC, 0, 0, n32W, n32H);
  XSetForeground(ptDisp, tGC, 0xFFFFFFFF);
  wcP.byte1 = un8Page;
  for(n32F = 0; n32F < 4; n32F++) {
    if(ptFont[n32F]) {
      XSetFont(ptDisp, tGC, ptFont[n32F]->fid);
      n32Pos = 0;
      for(n32Y = _YLOOPS_ * n32F; n32Y < (_YLOOPS_ * n32F + _YLOOPS_); n32Y++) {
        for(n32X = 0; n32X < _XLOOPS_; n32X++) {
          if((uint8_t(n32Pos) >= ptFont[n32F]->min_char_or_byte2) &&
          (uint8_t(n32Pos) <= ptFont[n32F]->max_char_or_byte2)) {
            wcP.byte2 = u_char(n32Pos);
            if(n32F & 0x01)
              XDrawString16(ptDisp, tPixmap, tGC, n32X * n32FW + 1, n32Y * n32FH + n32FA, &wcP, 1);
            else
              XDrawString16(ptDisp, tPixmap, tGC, n32X * n32FW, n32Y * n32FH + n32FA, &wcP, 1);
          }
          n32Pos++;
          if(n32Pos > 255) n32Pos = 255;
        }
      }
    }
  }

  // Main loop
  bExit = false;
  while(!bExit) {
    // Handle events
    if(XPending(ptDisp)) {
      XNextEvent(ptDisp, &tEvent);
      switch(tEvent.type) {
        case Expose:
          XCopyArea(ptDisp, tPixmap, tWin, tGC, 0, 0, n32W, n32H, 0, 0);
          break;
        case ButtonPress:
          bExit = true;
          break;
        default:
          break;
      }
    }
  }

  // Clean up
  XFreePixmap(ptDisp, tPixmap);
  XFreeGC(ptDisp, tGC);
  XUnmapWindow(ptDisp, tWin);
  XDestroyWindow(ptDisp, tWin);
  if(ptFont[0]) XFreeFont(ptDisp, ptFont[0]);
  if(ptFont[1]) XFreeFont(ptDisp, ptFont[1]);
  if(ptFont[2]) XFreeFont(ptDisp, ptFont[2]);
  if(ptFont[3]) XFreeFont(ptDisp, ptFont[3]);
  XCloseDisplay(ptDisp);
}


// Main program
int main()
{
  // Temporary variables
  uint8_t un8C;

  // Draw fonts
  //       If some (or all) are failed, try to change the font families and registrations
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("fixed", "iso8859", un8C, 0);
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("times", "iso8859", un8C, 0);
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("courier", "iso8859", un8C, 0);
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("lucida", "iso8859", un8C, 0);
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("mincho", "jisx0208.1983", un8C, 36);
  for(un8C = 1; un8C <= 16; un8C++) DrawFont("mincho", "jisx0208.1983", un8C, 37);

  // OK !
  return(0);
}

#endif
