// ############################################################################################
//  roomedit-DlgPages-Main-Common.cc
//  Implementation of roomedit's CPage_Main class (plane)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-DlgPages-Main.h"
#include "roomedit-RoomEdit.h"

// C++ headers
#include <sstream>


//
// Our main dialog page: operations section - objects (plane)
//
void CPage_Main::_onBtnNewPlane(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _app.pageNPS.SetPlaneSettings(NULL);
  _app.pageNPS.SetFuncToInvoke(&CPage_Main::_onBtnNewPlane);
  _app.SelectPage(CRoomEdit::PAGE_NPS);
}

void CPage_Main::_onBtnNewPlane(const TInit_PlaneSettings &init)
{
  _iVideoEnv->UseDefaultWaitCursor();

  _roomBuilder.NewPlane(init.planeName, _loc.curPos, init.planeRowSize, init.planeColSize, init.planeDir);
  lstObjects.AddItem(init.planeName);
  stable_sort(lstObjects.GetAllItems().begin(), lstObjects.GetAllItems().end());
  lstObjects.SelectItem(init.planeName);
  lstObjects.DrawControl();
  MarkAsDirty();

  _iVideoEnv->UseDefaultArrowCursor();
}

void CPage_Main::_onRdoMovCurObj_Plane()
{
  // Get the plane's name
  const char *planeName = lstObjects.GetSelectedItem();
  if(!planeName) return;

  // Move the cursor to the plane's origin
  const CPlane3D &sp = _roomBuilder.Plane(planeName);
  sp.GetOriginPosition(_loc.curPos);
}

void CPage_Main::_onBtnEditObject_Plane()
{
  // Get the plane's name
  const char *planeName = lstObjects.GetSelectedItem();
  if(!planeName) return;

  // Prepare the settings
  TInit_PlaneSettings curset;
  curset.planeName    = planeName;
  curset.planeRowSize = _roomBuilder.Plane(planeName).GetRowSize();
  curset.planeColSize = _roomBuilder.Plane(planeName).GetColSize();
  curset.planeDir     = _roomBuilder.Plane(planeName).GetDirection();

  // Show the dialog
  _app.pageNPS.SetPlaneSettings(&curset);
  _app.pageNPS.SetFuncToInvoke(&CPage_Main::_onBtnEditObject_Plane);
  _app.SelectPage(CRoomEdit::PAGE_NPS);
}

void CPage_Main::_onBtnEditObject_Plane(const TInit_PlaneSettings &init)
{
  _iVideoEnv->UseDefaultWaitCursor();

  TFloatXYZ org;
  _roomBuilder.Plane(init.planeName).GetOriginPosition(org);

  _roomBuilder.DelPlane(init.planeName);
  _roomBuilder.NewPlane(init.planeName, org, init.planeRowSize, init.planeColSize, init.planeDir);
  MarkAsDirty();

  _iVideoEnv->UseDefaultArrowCursor();
}

void CPage_Main::_onBtnRenObject_Plane()
{
  // Get the plane's name
  const char *planeName = lstObjects.GetSelectedItem();
  if(!planeName) return;

  // Get the new name
  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  const string &nn = _iVideoEnv->InputBox("Rename Plane", planeName);
  _vStrg.RestoreCurrentView();
  if(nn.empty() || nn==planeName) return;

  // Check if the new name already exist
  if(PlaneIsExist(nn)) {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Invalid New Name"), _M("A plane with the same name is already exist"),
                          EMBI_Error, EMBB_OK, EMBS_Tiny);
    _vStrg.RestoreCurrentView();
    return;
  }

  // Rename the plane
  _roomBuilder.RenamePlane(planeName, nn);
  lstObjects.DelItem(planeName);
  lstObjects.AddItem(nn);
  stable_sort(lstObjects.GetAllItems().begin(), lstObjects.GetAllItems().end());
  lstObjects.SelectItem(nn);
  lstObjects.DrawControl();
  MarkAsDirty();
}

void CPage_Main::_onBtnDelObject_Plane()
{
  // Get the plane's name
  const char *planeName = lstObjects.GetSelectedItem();
  if(!planeName) return;

  // Confirm deletion
  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  stringstream ss;
  ss << _M("Selected plane:")     << endl
     << "\"" << planeName << "\"" << endl
     << _M("delete it?");
  const EMsgBoxRetVal r = _iVideoEnv->MessageBox(_M("Delete Plane"), ss.str(),
                                                EMBI_Question, EMBB_YesNo, EMBS_Tiny);
  _vStrg.RestoreCurrentView();
  if(r == EMBRV_No) return;

  // Remove the plane
  _roomBuilder.DelPlane(planeName);
  lstObjects.DelItem(planeName);
  lstObjects.DrawControl();
  MarkAsDirty();
}

void CPage_Main::_onMove_Plane(const TFloatXYZ &mf)
{
  const char *planeName = lstObjects.GetSelectedItem();
  if(planeName) _roomBuilder.TranslatePlane(planeName, mf);
}
