// ############################################################################################
//  roomedit-RoomBuilder.h
//  Definition & implementation of roomedit's CRoomBuilder class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _UTIL___ROOMEDIT_ROOMBUILDER_H_
#define _UTIL___ROOMEDIT_ROOMBUILDER_H_

// VortexGE headers
#include <VMain.h>


//
// Our room builder class
//
class CRoomBuilder {
  public:
    CRoomBuilder();

    // Room management functions
    void NewRoom();
    void NewRoom(const TFloatXYZ &gridSize);
    void CloseRoom();
    inline void SetRoomGridSize(const TFloatXYZ &gridSize)
    { _pBuilder.SetGridSize(gridSize); }

    // Plane management functions
    void NewPlane(const string &planeName, const TFloatXYZ &planeOrigin, uint32_t planeRowSize, uint32_t planeColSize, EPlaneDir planeDir);
    void RenamePlane(const string &oldPlaneName, const string &newPlaneName);
    void DelPlane(const string &planeName);
    void TranslatePlane(const string &planeName, const TFloatXYZ &factor);

    // Non modifying access to the planes
    const CPlane3D &Plane(const string &planeName);

    // Rendering function
    void RenderAll(const CMatrix3D &mat, const string &selectedObject);

    // Information functions
    inline bool IsOpen() const
    { return(_open); }
    inline const TFloatXYZ &GetRoomGridSize() const
    { return(_pBuilder.GetGridSize()); }

  protected:
    typedef map<string, CPanel3DBuffer*> CP3DBufferMap;

    bool            _open;     // Flag to indicate the the class is open for room editing
    CPlane3DBuilder _pBuilder; // Plane 3D builder to be used to build the room
    CP3DBufferMap   _pBuffers; // Map of CPlane3DBuilder instance to be used for rendering the room
};

#endif
